@@ -38,11 +38,13 @@ fn lex_(mut input: &str, mut start_of_line: bool) -> impl Iterator<Item = (Synta
             match c {
                 ':' if colon_count == 0 => {
                     colon_count += 1;
-                    input = &input[1..];
+                    let char_len = c.len_utf8();
+                    input = &input[char_len..];
                     Some((SyntaxKind::COLON, ":"))
                 }
                 _ if common::is_newline(c) => {
-                    let (nl, remaining) = input.split_at(1);
+                    let char_len = c.len_utf8();
+                    let (nl, remaining) = input.split_at(char_len);
                     input = remaining;
                     start_of_line = true;
                     colon_count = 0;
@@ -85,7 +87,8 @@ fn lex_(mut input: &str, mut start_of_line: bool) -> impl Iterator<Item = (Synta
                     Some((SyntaxKind::VALUE, value))
                 }
                 _ => {
-                    let (text, remaining) = input.split_at(1);
+                    let char_len = c.len_utf8();
+                    let (text, remaining) = input.split_at(char_len);
                     input = remaining;
                     Some((SyntaxKind::ERROR, text))
                 }
