@@ -42,6 +42,57 @@ use rowan::ast::AstNode;
 use std::path::Path;
 use std::str::FromStr;
 
+/// Canonical field order for source paragraphs in debian/control files
+pub const SOURCE_FIELD_ORDER: &[&str] = &[
+    "Source",
+    "Section",
+    "Priority",
+    "Maintainer",
+    "Uploaders",
+    "Build-Depends",
+    "Build-Depends-Indep",
+    "Build-Depends-Arch",
+    "Build-Conflicts",
+    "Build-Conflicts-Indep",
+    "Build-Conflicts-Arch",
+    "Standards-Version",
+    "Vcs-Browser",
+    "Vcs-Git",
+    "Vcs-Svn",
+    "Vcs-Bzr",
+    "Vcs-Hg",
+    "Vcs-Darcs",
+    "Vcs-Cvs",
+    "Vcs-Arch",
+    "Vcs-Mtn",
+    "Homepage",
+    "Rules-Requires-Root",
+    "Testsuite",
+    "Testsuite-Triggers",
+];
+
+/// Canonical field order for binary packages in debian/control files
+pub const BINARY_FIELD_ORDER: &[&str] = &[
+    "Package",
+    "Architecture",
+    "Section",
+    "Priority",
+    "Multi-Arch",
+    "Essential",
+    "Build-Profiles",
+    "Built-Using",
+    "Pre-Depends",
+    "Depends",
+    "Recommends",
+    "Suggests",
+    "Enhances",
+    "Conflicts",
+    "Breaks",
+    "Replaces",
+    "Provides",
+    "Description",
+];
+
 /// A positioned parse error containing location information.
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub struct PositionedParseError {
@@ -161,7 +212,47 @@ pub(crate) fn parse(text: &str) -> Parse {
     }
 
     impl<'a> Parser<'a> {
+        /// Skip to next paragraph boundary for error recovery
+        fn skip_to_paragraph_boundary(&mut self) {
+            while self.current().is_some() {
+                match self.current() {
+                    Some(NEWLINE) => {
+                        self.bump();
+                        // Check if next line starts a new paragraph (key at start of line)
+                        if self.at_paragraph_start() {
+                            break;
+                        }
+                    }
+                    _ => {
+                        self.bump();
+                    }
+                }
+            }
+        }
+
+        /// Check if we're at the start of a new paragraph
+        fn at_paragraph_start(&self) -> bool {
+            match self.current() {
+                Some(KEY) => true,
+                Some(COMMENT) => true,
+                None => true, // EOF is a valid paragraph boundary
+                _ => false,
+            }
+        }
+
+        /// Attempt to recover from entry parsing errors
+        fn recover_entry(&mut self) {
+            // Skip to end of current line
+            while self.current().is_some() && self.current() != Some(NEWLINE) {
+                self.bump();
+            }
+            // Consume the newline if present
+            if self.current() == Some(NEWLINE) {
+                self.bump();
+            }
+        }
         fn parse_entry(&mut self) {
+            // Handle leading comments
             while self.current() == Some(COMMENT) {
                 self.bump();
 
@@ -175,46 +266,120 @@ pub(crate) fn parse(text: &str) -> Parse {
                     Some(g) => {
                         self.builder.start_node(ERROR.into());
                         self.add_positioned_error(
-                            format!("expected newline, got {g:?}"),
-                            Some("unexpected_token".to_string()),
+                            format!("expected newline after comment, got {g:?}"),
+                            Some("unexpected_token_after_comment".to_string()),
                         );
                         self.bump();
                         self.builder.finish_node();
+                        self.recover_entry();
+                        return;
                     }
                 }
             }
 
             self.builder.start_node(ENTRY.into());
+            let mut entry_has_errors = false;
 
-            // First, parse the key and colon
+            // Parse the key
             if self.current() == Some(KEY) {
                 self.bump();
                 self.skip_ws();
             } else {
+                entry_has_errors = true;
                 self.builder.start_node(ERROR.into());
-                self.add_positioned_error(
-                    "expected key".to_string(),
-                    Some("missing_key".to_string()),
-                );
-                if self.current().is_some() {
-                    self.bump();
+
+                // Enhanced error recovery for malformed keys
+                match self.current() {
+                    Some(VALUE) | Some(WHITESPACE) => {
+                        self.add_positioned_error(
+                            "field name cannot start with whitespace or special characters"
+                                .to_string(),
+                            Some("invalid_field_name".to_string()),
+                        );
+                        // Try to consume what might be an intended key
+                        while self.current() == Some(VALUE) || self.current() == Some(WHITESPACE) {
+                            self.bump();
+                        }
+                    }
+                    Some(COLON) => {
+                        self.add_positioned_error(
+                            "field name missing before colon".to_string(),
+                            Some("missing_field_name".to_string()),
+                        );
+                    }
+                    Some(NEWLINE) => {
+                        self.add_positioned_error(
+                            "empty line where field expected".to_string(),
+                            Some("empty_field_line".to_string()),
+                        );
+                        self.builder.finish_node();
+                        self.builder.finish_node();
+                        return;
+                    }
+                    _ => {
+                        self.add_positioned_error(
+                            format!("expected field name, got {:?}", self.current()),
+                            Some("missing_key".to_string()),
+                        );
+                        if self.current().is_some() {
+                            self.bump();
+                        }
+                    }
                 }
                 self.builder.finish_node();
             }
+
+            // Parse the colon
             if self.current() == Some(COLON) {
                 self.bump();
                 self.skip_ws();
             } else {
+                entry_has_errors = true;
                 self.builder.start_node(ERROR.into());
-                self.add_positioned_error(
-                    format!("expected ':', got {:?}", self.current()),
-                    Some("missing_colon".to_string()),
-                );
-                if self.current().is_some() {
-                    self.bump();
+
+                // Enhanced error recovery for missing colon
+                match self.current() {
+                    Some(VALUE) => {
+                        self.add_positioned_error(
+                            "missing colon ':' after field name".to_string(),
+                            Some("missing_colon".to_string()),
+                        );
+                        // Don't consume the value, let it be parsed as the field value
+                    }
+                    Some(NEWLINE) => {
+                        self.add_positioned_error(
+                            "field name without value (missing colon and value)".to_string(),
+                            Some("incomplete_field".to_string()),
+                        );
+                        self.builder.finish_node();
+                        self.builder.finish_node();
+                        return;
+                    }
+                    Some(KEY) => {
+                        self.add_positioned_error(
+                            "field name followed by another field name (missing colon and value)"
+                                .to_string(),
+                            Some("consecutive_field_names".to_string()),
+                        );
+                        // Don't consume the next key, let it be parsed as a new entry
+                        self.builder.finish_node();
+                        self.builder.finish_node();
+                        return;
+                    }
+                    _ => {
+                        self.add_positioned_error(
+                            format!("expected colon ':', got {:?}", self.current()),
+                            Some("missing_colon".to_string()),
+                        );
+                        if self.current().is_some() {
+                            self.bump();
+                        }
+                    }
                 }
                 self.builder.finish_node();
             }
+
+            // Parse the value (potentially multi-line)
             loop {
                 while self.current() == Some(WHITESPACE) || self.current() == Some(VALUE) {
                     self.bump();
@@ -227,16 +392,22 @@ pub(crate) fn parse(text: &str) -> Parse {
                     Some(NEWLINE) => {
                         self.bump();
                     }
+                    Some(KEY) => {
+                        // We've hit another field, this entry is complete
+                        break;
+                    }
                     Some(g) => {
                         self.builder.start_node(ERROR.into());
                         self.add_positioned_error(
-                            format!("expected newline, got {g:?}"),
-                            Some("unexpected_token".to_string()),
+                            format!("unexpected token in field value: {g:?}"),
+                            Some("unexpected_value_token".to_string()),
                         );
                         self.bump();
                         self.builder.finish_node();
                     }
                 }
+
+                // Check for continuation lines
                 if self.current() == Some(INDENT) {
                     self.bump();
                     self.skip_ws();
@@ -244,14 +415,91 @@ pub(crate) fn parse(text: &str) -> Parse {
                     break;
                 }
             }
+
             self.builder.finish_node();
+
+            // If the entry had errors, we might want to recover
+            if entry_has_errors && !self.at_paragraph_start() && self.current().is_some() {
+                self.recover_entry();
+            }
         }
 
         fn parse_paragraph(&mut self) {
             self.builder.start_node(PARAGRAPH.into());
+
+            let mut consecutive_errors = 0;
+            const MAX_CONSECUTIVE_ERRORS: usize = 5;
+
             while self.current() != Some(NEWLINE) && self.current().is_some() {
-                self.parse_entry();
+                let error_count_before = self.positioned_errors.len();
+
+                // Check if we're at a valid entry start
+                if self.current() == Some(KEY) || self.current() == Some(COMMENT) {
+                    self.parse_entry();
+
+                    // Reset consecutive error count if we successfully parsed something
+                    if self.positioned_errors.len() == error_count_before {
+                        consecutive_errors = 0;
+                    } else {
+                        consecutive_errors += 1;
+                    }
+                } else {
+                    // We're not at a valid entry start, this is an error
+                    consecutive_errors += 1;
+
+                    self.builder.start_node(ERROR.into());
+                    match self.current() {
+                        Some(VALUE) => {
+                            self.add_positioned_error(
+                                "orphaned text without field name".to_string(),
+                                Some("orphaned_text".to_string()),
+                            );
+                            // Consume the orphaned text
+                            while self.current() == Some(VALUE)
+                                || self.current() == Some(WHITESPACE)
+                            {
+                                self.bump();
+                            }
+                        }
+                        Some(COLON) => {
+                            self.add_positioned_error(
+                                "orphaned colon without field name".to_string(),
+                                Some("orphaned_colon".to_string()),
+                            );
+                            self.bump();
+                        }
+                        Some(INDENT) => {
+                            self.add_positioned_error(
+                                "unexpected indentation without field".to_string(),
+                                Some("unexpected_indent".to_string()),
+                            );
+                            self.bump();
+                        }
+                        _ => {
+                            self.add_positioned_error(
+                                format!(
+                                    "unexpected token at paragraph level: {:?}",
+                                    self.current()
+                                ),
+                                Some("unexpected_paragraph_token".to_string()),
+                            );
+                            self.bump();
+                        }
+                    }
+                    self.builder.finish_node();
+                }
+
+                // If we have too many consecutive errors, skip to paragraph boundary
+                if consecutive_errors >= MAX_CONSECUTIVE_ERRORS {
+                    self.add_positioned_error(
+                        "too many consecutive parse errors, skipping to next paragraph".to_string(),
+                        Some("parse_recovery".to_string()),
+                    );
+                    self.skip_to_paragraph_boundary();
+                    break;
+                }
             }
+
             self.builder.finish_node();
         }
 
@@ -906,10 +1154,11 @@ impl Paragraph {
         self.0.splice_children(count..count, vec![entry.0.into()]);
     }
 
-    /// Set a field in the paragraph
+    /// Set a field in the paragraph, inserting at the appropriate location if new
     pub fn set(&mut self, key: &str, value: &str) {
         let new_entry = Entry::new(key, value);
 
+        // Check if the field already exists and replace it
         for entry in self.entries() {
             if entry.key().as_deref() == Some(key) {
                 self.0.splice_children(
@@ -919,9 +1168,128 @@ impl Paragraph {
                 return;
             }
         }
-        let count = self.0.children_with_tokens().count();
+
+        // Field doesn't exist, insert at appropriate location
+        // Try to detect if this is a source or binary package paragraph
+        let field_order = if self.contains_key("Source") {
+            SOURCE_FIELD_ORDER
+        } else if self.contains_key("Package") {
+            BINARY_FIELD_ORDER
+        } else {
+            // Default to trying both, preferring source if the new field is "Source"
+            if key == "Source" {
+                SOURCE_FIELD_ORDER
+            } else if key == "Package" {
+                BINARY_FIELD_ORDER
+            } else {
+                // Try to determine based on existing fields
+                let has_source_fields = self.keys().any(|k| {
+                    SOURCE_FIELD_ORDER.contains(&k.as_str())
+                        && !BINARY_FIELD_ORDER.contains(&k.as_str())
+                });
+                if has_source_fields {
+                    SOURCE_FIELD_ORDER
+                } else {
+                    BINARY_FIELD_ORDER
+                }
+            }
+        };
+
+        let insertion_index = self.find_insertion_index(key, field_order);
         self.0
-            .splice_children(count..count, vec![new_entry.0.into()]);
+            .splice_children(insertion_index..insertion_index, vec![new_entry.0.into()]);
+    }
+
+    /// Set a field using a specific field ordering
+    pub fn set_with_field_order(&mut self, key: &str, value: &str, field_order: &[&str]) {
+        let new_entry = Entry::new(key, value);
+
+        // Check if the field already exists and replace it
+        for entry in self.entries() {
+            if entry.key().as_deref() == Some(key) {
+                self.0.splice_children(
+                    entry.0.index()..entry.0.index() + 1,
+                    vec![new_entry.0.into()],
+                );
+                return;
+            }
+        }
+
+        let insertion_index = self.find_insertion_index(key, field_order);
+        self.0
+            .splice_children(insertion_index..insertion_index, vec![new_entry.0.into()]);
+    }
+
+    /// Find the appropriate insertion index for a new field based on field ordering
+    fn find_insertion_index(&self, key: &str, field_order: &[&str]) -> usize {
+        // Find position of the new field in the canonical order
+        let new_field_position = field_order.iter().position(|&field| field == key);
+
+        let mut insertion_index = self.0.children_with_tokens().count();
+
+        // Find the right position based on canonical field order
+        for (i, child) in self.0.children_with_tokens().enumerate() {
+            if let Some(node) = child.as_node() {
+                if let Some(entry) = Entry::cast(node.clone()) {
+                    if let Some(existing_key) = entry.key() {
+                        let existing_position =
+                            field_order.iter().position(|&field| field == existing_key);
+
+                        match (new_field_position, existing_position) {
+                            // Both fields are in the canonical order
+                            (Some(new_pos), Some(existing_pos)) => {
+                                if new_pos < existing_pos {
+                                    insertion_index = i;
+                                    break;
+                                }
+                            }
+                            // New field is in canonical order, existing is not
+                            (Some(_), None) => {
+                                // Continue looking - unknown fields go after known ones
+                            }
+                            // New field is not in canonical order, existing is
+                            (None, Some(_)) => {
+                                // Continue until we find all known fields
+                            }
+                            // Neither field is in canonical order, maintain alphabetical
+                            (None, None) => {
+                                if key < existing_key.as_str() {
+                                    insertion_index = i;
+                                    break;
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        }
+
+        // If we have a position in canonical order but haven't found where to insert yet,
+        // we need to insert after all known fields that come before it
+        if new_field_position.is_some() && insertion_index == self.0.children_with_tokens().count()
+        {
+            // Look for the position after the last known field that comes before our field
+            let children: Vec<_> = self.0.children_with_tokens().enumerate().collect();
+            for (i, child) in children.into_iter().rev() {
+                if let Some(node) = child.as_node() {
+                    if let Some(entry) = Entry::cast(node.clone()) {
+                        if let Some(existing_key) = entry.key() {
+                            if field_order
+                                .iter()
+                                .position(|&f| f == existing_key)
+                                .is_some()
+                            {
+                                // Found a known field, insert after it
+                                insertion_index = i + 1;
+                                break;
+                            }
+                        }
+                    }
+                }
+            }
+        }
+
+        insertion_index
     }
 
     /// Rename the given field in the paragraph.
@@ -996,12 +1364,14 @@ impl<'a, 'py> pyo3::IntoPyObject<'py> for &'a Paragraph {
 }
 
 #[cfg(feature = "python-debian")]
-impl pyo3::FromPyObject<'_> for Paragraph {
-    fn extract_bound(obj: &pyo3::Bound<pyo3::PyAny>) -> pyo3::PyResult<Self> {
-        use pyo3::prelude::*;
+impl<'py> pyo3::FromPyObject<'_, 'py> for Paragraph {
+    type Error = pyo3::PyErr;
+
+    fn extract(obj: pyo3::Borrowed<'_, 'py, pyo3::PyAny>) -> Result<Self, Self::Error> {
+        use pyo3::types::PyAnyMethods;
         let d = obj.call_method0("__str__")?.extract::<String>()?;
-        Ok(Paragraph::from_str(&d)
-            .map_err(|e| pyo3::exceptions::PyValueError::new_err((e.to_string(),)))?)
+        Paragraph::from_str(&d)
+            .map_err(|e| pyo3::exceptions::PyValueError::new_err((e.to_string(),)))
     }
 }
 
@@ -1924,6 +2294,83 @@ C: D
         assert_eq!(ParseError(vec!["foo".to_string()]).to_string(), "foo\n");
     }
 
+    #[test]
+    fn test_set_field_ordering_source() {
+        let mut p = super::Paragraph::new();
+
+        // Add fields in random order
+        p.set("Homepage", "https://example.com");
+        p.set("Source", "mypackage");
+        p.set("Build-Depends", "debhelper");
+        p.set("Maintainer", "Test <test@example.com>");
+        p.set("Standards-Version", "4.5.0");
+
+        // Check that fields are in the expected order
+        let keys: Vec<_> = p.keys().collect();
+        assert_eq!(keys[0], "Source");
+        assert_eq!(keys[1], "Maintainer");
+        assert_eq!(keys[2], "Build-Depends");
+        assert_eq!(keys[3], "Standards-Version");
+        assert_eq!(keys[4], "Homepage");
+    }
+
+    #[test]
+    fn test_set_field_ordering_binary() {
+        let mut p = super::Paragraph::new();
+
+        // Add fields in random order
+        p.set("Description", "A test package");
+        p.set("Package", "mypackage");
+        p.set("Depends", "libc6");
+        p.set("Architecture", "amd64");
+        p.set("Section", "utils");
+
+        // Check that fields are in the expected order
+        let keys: Vec<_> = p.keys().collect();
+        assert_eq!(keys[0], "Package");
+        assert_eq!(keys[1], "Architecture");
+        assert_eq!(keys[2], "Section");
+        assert_eq!(keys[3], "Depends");
+        assert_eq!(keys[4], "Description");
+    }
+
+    #[test]
+    fn test_set_field_ordering_mixed() {
+        let mut p = super::Paragraph::new();
+
+        // Add some known fields and some unknown fields
+        p.set("Package", "mypackage");
+        p.set("X-Custom-Field", "custom");
+        p.set("Architecture", "all");
+        p.set("Another-Custom", "value");
+        p.set("Description", "Test");
+
+        // Check that known fields are ordered correctly and unknown fields are alphabetical
+        let keys: Vec<_> = p.keys().collect();
+        assert_eq!(keys[0], "Package");
+        assert_eq!(keys[1], "Architecture");
+        assert_eq!(keys[2], "Description");
+        assert_eq!(keys[3], "Another-Custom"); // Unknown fields alphabetically sorted
+        assert_eq!(keys[4], "X-Custom-Field");
+    }
+
+    #[test]
+    fn test_set_with_field_order() {
+        let mut p = super::Paragraph::new();
+        let custom_order = &["Foo", "Bar", "Baz"];
+
+        p.set_with_field_order("Baz", "3", custom_order);
+        p.set_with_field_order("Foo", "1", custom_order);
+        p.set_with_field_order("Bar", "2", custom_order);
+        p.set_with_field_order("Unknown", "4", custom_order);
+
+        let keys: Vec<_> = p.keys().collect();
+        assert_eq!(keys[0], "Foo");
+        assert_eq!(keys[1], "Bar");
+        assert_eq!(keys[2], "Baz");
+        assert_eq!(keys[3], "Unknown");
+    }
+
     #[test]
     fn test_positioned_parse_error() {
         let error = PositionedParseError {
@@ -2104,4 +2551,248 @@ Version: 1.0"#;
         assert!(key_range.start() >= text_range.start());
         assert!(value_range.end() <= text_range.end());
     }
+
+    #[test]
+    fn test_error_recovery_missing_colon() {
+        let input = r#"Source foo
+Maintainer: Test User <test@example.com>
+"#;
+        let (deb822, errors) = super::Deb822::from_str_relaxed(input);
+
+        // Should still parse successfully with errors
+        assert!(!errors.is_empty());
+        assert!(errors.iter().any(|e| e.contains("missing colon")));
+
+        // Should still have a paragraph with the valid field
+        let paragraph = deb822.paragraphs().next().unwrap();
+        assert_eq!(
+            paragraph.get("Maintainer").as_deref(),
+            Some("Test User <test@example.com>")
+        );
+    }
+
+    #[test]
+    fn test_error_recovery_missing_field_name() {
+        let input = r#": orphaned value
+Package: test
+"#;
+
+        let (deb822, errors) = super::Deb822::from_str_relaxed(input);
+
+        // Should have errors about missing field name
+        assert!(!errors.is_empty());
+        assert!(errors
+            .iter()
+            .any(|e| e.contains("field name") || e.contains("missing")));
+
+        // The valid field should be in one of the paragraphs
+        let paragraphs: Vec<_> = deb822.paragraphs().collect();
+        let mut found_package = false;
+        for paragraph in paragraphs.iter() {
+            if paragraph.get("Package").is_some() {
+                found_package = true;
+                assert_eq!(paragraph.get("Package").as_deref(), Some("test"));
+            }
+        }
+        assert!(found_package, "Package field not found in any paragraph");
+    }
+
+    #[test]
+    fn test_error_recovery_orphaned_text() {
+        let input = r#"Package: test
+some orphaned text without field name
+Version: 1.0
+"#;
+        let (deb822, errors) = super::Deb822::from_str_relaxed(input);
+
+        // Should have errors about orphaned text
+        assert!(!errors.is_empty());
+        assert!(errors.iter().any(|e| e.contains("orphaned")
+            || e.contains("unexpected")
+            || e.contains("field name")));
+
+        // Should still parse the valid fields (may be split across paragraphs)
+        let mut all_fields = std::collections::HashMap::new();
+        for paragraph in deb822.paragraphs() {
+            for (key, value) in paragraph.items() {
+                all_fields.insert(key, value);
+            }
+        }
+
+        assert_eq!(all_fields.get("Package"), Some(&"test".to_string()));
+        assert_eq!(all_fields.get("Version"), Some(&"1.0".to_string()));
+    }
+
+    #[test]
+    fn test_error_recovery_consecutive_field_names() {
+        let input = r#"Package: test
+Description
+Maintainer: Another field without proper value
+Version: 1.0
+"#;
+        let (deb822, errors) = super::Deb822::from_str_relaxed(input);
+
+        // Should have errors about missing values
+        assert!(!errors.is_empty());
+        assert!(errors.iter().any(|e| e.contains("consecutive")
+            || e.contains("missing")
+            || e.contains("incomplete")));
+
+        // Should still parse valid fields (may be split across paragraphs due to errors)
+        let mut all_fields = std::collections::HashMap::new();
+        for paragraph in deb822.paragraphs() {
+            for (key, value) in paragraph.items() {
+                all_fields.insert(key, value);
+            }
+        }
+
+        assert_eq!(all_fields.get("Package"), Some(&"test".to_string()));
+        assert_eq!(
+            all_fields.get("Maintainer"),
+            Some(&"Another field without proper value".to_string())
+        );
+        assert_eq!(all_fields.get("Version"), Some(&"1.0".to_string()));
+    }
+
+    #[test]
+    fn test_error_recovery_malformed_multiline() {
+        let input = r#"Package: test
+Description: Short desc
+  Proper continuation
+invalid continuation without indent
+ Another proper continuation
+Version: 1.0
+"#;
+        let (deb822, errors) = super::Deb822::from_str_relaxed(input);
+
+        // Should recover from malformed continuation
+        assert!(!errors.is_empty());
+
+        // Should still parse other fields correctly
+        let paragraph = deb822.paragraphs().next().unwrap();
+        assert_eq!(paragraph.get("Package").as_deref(), Some("test"));
+        assert_eq!(paragraph.get("Version").as_deref(), Some("1.0"));
+    }
+
+    #[test]
+    fn test_error_recovery_mixed_errors() {
+        let input = r#"Package test without colon
+: orphaned colon
+Description: Valid field
+some orphaned text
+Another-Field: Valid too
+"#;
+        let (deb822, errors) = super::Deb822::from_str_relaxed(input);
+
+        // Should have multiple different errors
+        assert!(!errors.is_empty());
+        assert!(errors.len() >= 2);
+
+        // Should still parse the valid fields
+        let paragraph = deb822.paragraphs().next().unwrap();
+        assert_eq!(paragraph.get("Description").as_deref(), Some("Valid field"));
+        assert_eq!(paragraph.get("Another-Field").as_deref(), Some("Valid too"));
+    }
+
+    #[test]
+    fn test_error_recovery_paragraph_boundary() {
+        let input = r#"Package: first-package
+Description: First paragraph
+
+corrupted data here
+: more corruption
+completely broken line
+
+Package: second-package
+Version: 1.0
+"#;
+        let (deb822, errors) = super::Deb822::from_str_relaxed(input);
+
+        // Should have errors from the corrupted section
+        assert!(!errors.is_empty());
+
+        // Should still parse both paragraphs correctly
+        let paragraphs: Vec<_> = deb822.paragraphs().collect();
+        assert_eq!(paragraphs.len(), 2);
+
+        assert_eq!(
+            paragraphs[0].get("Package").as_deref(),
+            Some("first-package")
+        );
+        assert_eq!(
+            paragraphs[1].get("Package").as_deref(),
+            Some("second-package")
+        );
+        assert_eq!(paragraphs[1].get("Version").as_deref(), Some("1.0"));
+    }
+
+    #[test]
+    fn test_error_recovery_with_positioned_errors() {
+        let input = r#"Package test
+Description: Valid
+"#;
+        let parsed = super::parse(input);
+
+        // Should have positioned errors with proper ranges
+        assert!(!parsed.positioned_errors.is_empty());
+
+        let first_error = &parsed.positioned_errors[0];
+        assert!(!first_error.message.is_empty());
+        assert!(first_error.range.start() <= first_error.range.end());
+        assert!(first_error.code.is_some());
+
+        // Error should point to the problematic location
+        let error_text = &input[first_error.range.start().into()..first_error.range.end().into()];
+        assert!(!error_text.is_empty());
+    }
+
+    #[test]
+    fn test_error_recovery_preserves_whitespace() {
+        let input = r#"Source: package
+Maintainer   Test User <test@example.com>
+Section:    utils
+
+"#;
+        let (deb822, errors) = super::Deb822::from_str_relaxed(input);
+
+        // Should have error about missing colon
+        assert!(!errors.is_empty());
+
+        // Should preserve original formatting in output
+        let output = deb822.to_string();
+        assert!(output.contains("Section:    utils"));
+
+        // Should still extract valid fields
+        let paragraph = deb822.paragraphs().next().unwrap();
+        assert_eq!(paragraph.get("Source").as_deref(), Some("package"));
+        assert_eq!(paragraph.get("Section").as_deref(), Some("utils"));
+    }
+
+    #[test]
+    fn test_error_recovery_empty_fields() {
+        let input = r#"Package: test
+Description:
+Maintainer: Valid User
+EmptyField: 
+Version: 1.0
+"#;
+        let (deb822, _errors) = super::Deb822::from_str_relaxed(input);
+
+        // Empty fields should parse without major errors - collect all fields from all paragraphs
+        let mut all_fields = std::collections::HashMap::new();
+        for paragraph in deb822.paragraphs() {
+            for (key, value) in paragraph.items() {
+                all_fields.insert(key, value);
+            }
+        }
+
+        assert_eq!(all_fields.get("Package"), Some(&"test".to_string()));
+        assert_eq!(all_fields.get("Description"), Some(&"".to_string()));
+        assert_eq!(
+            all_fields.get("Maintainer"),
+            Some(&"Valid User".to_string())
+        );
+        assert_eq!(all_fields.get("EmptyField"), Some(&"".to_string()));
+        assert_eq!(all_fields.get("Version"), Some(&"1.0".to_string()));
+    }
 }
